package ch.fritteli.maze.server;

import ch.fritteli.maze.generator.renderer.html.HTMLRenderer;
import ch.fritteli.maze.generator.renderer.json.JsonRenderer;
import ch.fritteli.maze.generator.renderer.pdf.PDFRenderer;
import ch.fritteli.maze.generator.renderer.text.TextRenderer;
import ch.fritteli.maze.generator.serialization.v1.SerializerDeserializerV1;
import ch.fritteli.maze.generator.serialization.v2.SerializerDeserializerV2;
import ch.fritteli.maze.generator.model.Maze;
import io.vavr.collection.List;
import io.vavr.collection.Stream;
import io.vavr.control.Option;
import lombok.Getter;
import lombok.NonNull;
import org.jetbrains.annotations.Nullable;

import java.nio.charset.StandardCharsets;
import java.util.function.Function;

public enum OutputType {
    TEXT_PLAIN("text/plain; charset=UTF-8",
            "txt",
            maze -> TextRenderer.newInstance().render(maze).getBytes(StandardCharsets.UTF_8),
            false,
            "t",
            "text"),
    HTML("text/html",
            "html",
            maze -> HTMLRenderer.newInstance().render(maze).getBytes(StandardCharsets.UTF_8),
            false,
            "h",
            "html"),
    JSON("application/json",
            "json",
            maze -> JsonRenderer.newInstance().render(maze).getBytes(StandardCharsets.UTF_8),
            false,
            "j",
            "json"),
    JSONFILE("application/json",
            "json",
            maze -> JsonRenderer.newInstance().render(maze).getBytes(StandardCharsets.UTF_8),
            true,
            "s",
            "jsonfile"),
    PDF("application/pdf",
            "pdf",
            maze -> PDFRenderer.newInstance().render(maze).toByteArray(),
            false,
            "p",
            "pdf"),
    PDFFILE("application/pdf",
            "pdf",
            maze -> PDFRenderer.newInstance().render(maze).toByteArray(),
            true,
            "f",
            "pdffile"),
    BINARY("application/octet-stream",
            "laby",
            SerializerDeserializerV1::serialize,
            true,
            "b",
            "binary"),
    BINARY_V2("application/octet-stream",
            "maze",
            SerializerDeserializerV2::serialize,
            true,
            "v",
            "binaryv2");
    @Getter
    @NonNull
    private final String contentType;
    @Getter
    @NonNull
    private final String fileExtension;
    @NonNull
    private final Function<Maze, byte[]> render;
    @Getter
    private final boolean attachment;
    @Getter
    @NonNull
    private final List<String> names;

    OutputType(@NonNull final String contentType,
               @NonNull final String fileExtension,
               @NonNull final Function<Maze, byte[]> render,
               final boolean attachment,
               @NonNull final String... names) {
        this.contentType = contentType;
        this.render = render;
        this.fileExtension = fileExtension;
        this.attachment = attachment;
        this.names = List.of(names);
    }

    @NonNull
    public static Option<OutputType> ofString(@Nullable final String name) {
        return Option.of(name)
                .map(String::toLowerCase)
                .flatMap(nameLC -> Stream.of(values())
                        .find(param -> param.names.contains(nameLC)));

    }

    @NonNull
    @Override
    public String toString() {
        return this.names.last();
    }

    @NonNull
    public byte[] render(@NonNull final Maze maze) {
        return this.render.apply(maze);
    }
}
