package ch.fritteli.maze.server;

import io.vavr.control.Try;
import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.NonNull;
import lombok.Value;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.Nullable;

import java.net.InetAddress;

@AllArgsConstructor(access = AccessLevel.PRIVATE)
@Slf4j
@Value
public class ServerConfig {
    public static final String SYSPROP_HOST = "fritteli.maze.server.host";
    public static final String SYSPROP_PORT = "fritteli.maze.server.port";

    @NonNull
    InetAddress address;
    int port;

    public ServerConfig(@Nullable final String address, final int port) throws ConfigurationException {
        this.address = validateAddress(address);
        this.port = validatePort(port);
        log.debug("host={}, port={}", this.address, this.port);
    }

    @NonNull
    public static ServerConfig init() throws ConfigurationException {
        final String host = System.getProperty(SYSPROP_HOST);
        final String portString = System.getProperty(SYSPROP_PORT);
        final int port = validatePort(portString);
        return new ServerConfig(host, port);
    }

    @NonNull
    private static InetAddress validateAddress(@Nullable final String address) {
        return Try.of(() -> InetAddress.getByName(address))
                .getOrElseThrow(cause -> new ConfigurationException(
                        "Invalid hostname/address: %s".formatted(address),
                        cause
                ));
    }

    private static int validatePort(final int port) {
        if (port < 0 || port > 0xFFFF) {
            throw new ConfigurationException("Port out of range (0..65535): %s".formatted(port));
        }
        return port;
    }

    private static int validatePort(@Nullable final String portString) {
        if (portString == null) {
            log.info("No port configured; using default.");
            return 0;
        }
        return Try.of(() -> Integer.valueOf(portString))
                .getOrElseThrow(cause -> new ConfigurationException(
                        "Failed to parse port specified in system property '%s': %s".formatted(SYSPROP_PORT, portString),
                        cause
                ));
    }
}
