package ch.fritteli.maze.server.handler;

import ch.fritteli.maze.server.OutputType;
import ch.fritteli.maze.generator.model.Maze;
import io.undertow.server.HttpServerExchange;
import io.undertow.util.Headers;
import io.undertow.util.HttpString;
import io.undertow.util.StatusCodes;
import io.vavr.Tuple2;
import io.vavr.control.Try;
import java.nio.ByteBuffer;
import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.Deque;
import java.util.Map;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.MDC;

@Slf4j
public class CreateHandler extends AbstractHttpHandler {

    public static final String PATH_TEMPLATE = "/create/{output}";

    @Override
    protected void handle(@NonNull final HttpServerExchange exchange) {
        final Instant start = Instant.now();
        log.debug("Handling create request");
        this.createMazeFromRequestParameters(exchange.getQueryParameters())
                .onFailure(e -> {
                    log.error("Error creating maze from request", e);
                    exchange.setStatusCode(StatusCodes.INTERNAL_SERVER_ERROR)
                            .setReasonPhrase(StatusCodes.INTERNAL_SERVER_ERROR_STRING)
                            .getResponseSender()
                            .send(e.getMessage());
                })
                .forEach(tuple -> {
                    final OutputType outputType = tuple._1();
                    final Maze maze = tuple._2();
                    final byte[] bytes;
                    try {
                        bytes = outputType.render(maze);
                    } catch (@NonNull final Exception e) {
                        log.error("Error rendering Maze", e);
                        exchange.setStatusCode(StatusCodes.INTERNAL_SERVER_ERROR)
                                .setReasonPhrase(StatusCodes.INTERNAL_SERVER_ERROR_STRING)
                                .getResponseSender()
                                .send("Error creating the maze. Please contact the administrator. Request id=%s".formatted(MDC.get("correlationId")));
                        return;
                    }
                    final long durationMillis = start.until(Instant.now(), ChronoUnit.MILLIS);
                    exchange.getResponseHeaders()
                            .put(Headers.CONTENT_TYPE, outputType.getContentType())
                            .put(HttpString.tryFromString("X-Maze-ID"), String.valueOf(maze.getRandomSeed()))
                            .put(HttpString.tryFromString("X-Maze-Width"), String.valueOf(maze.getWidth()))
                            .put(HttpString.tryFromString("X-Maze-Height"), String.valueOf(maze.getHeight()))
                            // TODO read the algorithm from the maze once this is supported.
                            .put(HttpString.tryFromString("X-Maze-Algorithm"), "RandomDepthFirst")
                            .put(HttpString.tryFromString("X-Maze-Generation-Duration-millis"), String.valueOf(durationMillis));
                    if (outputType.isAttachment()) {
                        exchange.getResponseHeaders()
                                .put(Headers.CONTENT_DISPOSITION, "attachment; filename=\"maze-%dx%d-%d.%s\"".formatted(
                                        maze.getWidth(),
                                        maze.getHeight(),
                                        maze.getRandomSeed(),
                                        outputType.getFileExtension()
                                ));
                    }
                    exchange.getResponseSender().send(ByteBuffer.wrap(bytes));
                    log.debug("Create request handled in {}ms.", durationMillis);
                });
    }

    @NonNull
    private Try<Tuple2<OutputType, Maze>> createMazeFromRequestParameters(final Map<String, Deque<String>> queryParameters) {
        return new ParametersToMazeExtractor(queryParameters).createMaze();
    }
}
