package ch.fritteli.maze.server.handler;

import ch.fritteli.maze.server.OutputType;
import ch.fritteli.maze.generator.algorithm.RandomDepthFirst;
import ch.fritteli.maze.generator.model.Maze;
import ch.fritteli.maze.generator.model.Position;
import io.vavr.Tuple;
import io.vavr.Tuple2;
import io.vavr.collection.Stream;
import io.vavr.control.Option;
import io.vavr.control.Try;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;

import java.util.Deque;
import java.util.Map;
import java.util.Random;

@RequiredArgsConstructor
class ParametersToMazeExtractor {

    @NonNull
    private final Map<String, Deque<String>> queryParameters;

    @NonNull
    Try<Tuple2<OutputType, Maze>> createMaze() {
        final Option<OutputType> output = getParameterValue(RequestParameter.OUTPUT);
        final Option<Integer> width = getParameterValue(RequestParameter.WIDTH);
        final Option<Integer> height = getParameterValue(RequestParameter.HEIGHT);
        final Option<Long> id = getParameterValue(RequestParameter.ID);
        final Option<Position> start = getParameterValue(RequestParameter.START);
        final Option<Position> end = getParameterValue(RequestParameter.END);

        if (output.isEmpty()) {
            return Try.failure(new IllegalArgumentException("Path parameter %s is required and must be one of: %s".formatted(
                    RequestParameter.OUTPUT.getNames().mkString("'", " / ", "'"),
                    Stream.of(OutputType.values())
                            .flatMap(OutputType::getNames)
                            .mkString(", ")
            )));
        }
        if (width.isEmpty()) {
            return Try.failure(new IllegalArgumentException("Query parameter %s is required and must be a positive integer value".formatted(
                    RequestParameter.WIDTH.getNames().mkString("'", " / ", "'")
            )));
        }
        if (height.isEmpty()) {
            return Try.failure(new IllegalArgumentException("Query parameter %s is required and must be a positive integer value".formatted(
                    RequestParameter.HEIGHT.getNames().mkString("'", " / ", "'")
            )));
        }

        return Try.of(() -> {
            final Maze maze;
            if (start.isDefined() && end.isDefined()) {
                maze = new Maze(width.get(), height.get(), id.getOrElse(() -> new Random().nextLong()), start.get(), end.get());
            } else {
                maze = new Maze(width.get(), height.get(), id.getOrElse(() -> new Random().nextLong()));
            }
            new RandomDepthFirst(maze).run();
            return Tuple.of(output.get(), maze);
        });
    }

    @NonNull
    private <T> Option<T> getParameterValue(@NonNull final RequestParameter parameter) {
        return parameter.getParameterValue(this.queryParameters);
    }
}
