package ch.fritteli.maze.server.handler;

import ch.fritteli.maze.server.OutputType;
import ch.fritteli.maze.generator.model.Position;
import io.vavr.Tuple2;
import io.vavr.collection.HashMap;
import io.vavr.collection.HashSet;
import io.vavr.collection.Set;
import io.vavr.control.Option;
import io.vavr.control.Try;
import lombok.Getter;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

import java.util.Deque;
import java.util.Map;
import java.util.function.Function;

@Slf4j
enum RequestParameter {
    WIDTH(p -> Try.of(() -> Integer.parseInt(p))
            .toOption()
            .onEmpty(() -> log.debug("Unparseable value for parameter 'width': '{}'", p)), "w", "width"),
    HEIGHT(p -> Try.of(() -> Integer.parseInt(p))
            .toOption()
            .onEmpty(() -> log.debug("Unparseable value for parameter 'height': '{}'", p)), "h", "height"),
    ID(p -> Try.of(() -> Long.parseLong(p))
            .toOption()
            .onEmpty(() -> log.debug("Unparseable value for parameter 'id': '{}'", p)), "i", "id"),
    OUTPUT(p -> OutputType.ofString(p)
            .onEmpty(() -> log.debug("Unparseable value for parameter 'output': '{}'", p)), "o", "output"),
    START(p -> Try.of(() -> {
                final String[] parts = p.split(",");
                final int x = Integer.parseInt(parts[0]);
                final int y = Integer.parseInt(parts[1]);
                return new Position(x, y);
            })
            .toOption()
            .onEmpty(() -> log.debug("Unparseable value for parameter 'start': '{}'", p)), "s", "start"),
    END(p -> Try.of(() -> {
                final String[] parts = p.split(",");
                final int x = Integer.parseInt(parts[0]);
                final int y = Integer.parseInt(parts[1]);
                return new Position(x, y);
            })
            .toOption()
            .onEmpty(() -> log.debug("Unparseable value for parameter 'end': '{}'", p)), "e", "end");
    @NonNull
    private final Function<String, Option<?>> extractor;
    @Getter
    @NonNull
    private final Set<String> names;

    RequestParameter(@NonNull final Function<String, Option<?>> extractor, @NonNull final String... names) {
        this.extractor = extractor;
        this.names = HashSet.of(names);
    }

    @NonNull
    Option<?> extractParameterValue(@NonNull final String parameter) {
        return this.extractor.apply(parameter);
    }

    @NonNull
    public <T> Option<T> getParameterValue(@NonNull final Map<String, Deque<String>> queryParameters) {
        return (Option<T>) HashMap.ofAll(queryParameters)
                .filterKeys(this.names::contains)
                .flatMap(Tuple2::_2)
                .flatMap(this::extractParameterValue)
                .headOption();
    }
}
